# Matrix Dart SDK

  [![](https://img.shields.io/pub/v/matrix_sdk.svg)](https://pub.dartlang.org/packages/matrix_sdk)
  [![](https://img.shields.io/matrix/sdk:pattle.im.svg?server_fqdn=matrix.org)](https://matrix.to/#/#sdk:matrix.org)

  A Dart implementation of the federated communications protocol, Matrix.

  This SDK is for general use, but mainly being developed for [Pattle](https://pattle.im), an
  easy to use Matrix app.
   
## Usage
   
   ```dart
   import 'package:matrix_sdk/matrix_sdk.dart';
   import 'package:url/url.dart';

   void main() async {
     final sqfliteStore = SqfliteStore();
     final homeserver = Homeserver(Url.parse('https://pattle.im'));

     final user = await homeserver.login(Username('pat'), 'pattle', store: sqfliteStore);

     await for(SyncState syncState in user.sync()) {
       await for (final room in user.rooms.all()) {
         // Do something
       }
     }
   }
   ```
   This example uses the
   [`SqfliteStore`](https://git.pattle.im/pattle/library/matrix-dart-sdk-sqflite)
   , which is a seperate dependency and only works on Flutter.

   Note that using a store is optional.

   Also uses the [`url`](https://pub.dev/packages/url) package for `Url.parse(..)`.

## Contributing

   To contribute, a [CLA](https://git.pattle.im/pattle/util/cla/blob/master/cla.pdf)
   has to be signed. This is the Fiduciary Contributor License Agreement,
   a very free software friendly license agreement. It actually guarantees you (the Contributor)
   that your contributions will never be relicensed to anything other than AGPL version 3 or
   higher.

   To sign the CLA, please send a signed copy to [wilko@rens.onl](mailto:wilko@rens.onl).

   An image of your signature is also okay, to do this easily you can edit the
   [HTML version](https://git.pattle.im/pattle/util/cla/blob/master/cla.html), add a
   photo of your signature as an `<img>` tag, render it with
   [`pandoc`](https://pandoc.org/) using:
   ```sh
   pandoc cla-signed.html -t html -o cla-signed.pdf --css cla.css
   ```

   Please note that in the future the signing process will be automated using
   [CLAM](https://gitlab.com/Xatom/CLAM).
