// Copyright (C) 2019  Wilko Manger
// Copyright (C) 2019  Mathieu Velten
//
// This file is part of matrix-dart-sdk.
//
// matrix-dart-sdk is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// matrix-dart-sdk is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with matrix-dart-sdk.  If not, see <https://www.gnu.org/licenses/>.
import 'dart:async';
import 'package:chopper/chopper.dart';
import 'package:matrix_sdk/src/api/api.dart';
import 'package:meta/meta.dart';

part 'client.chopper.dart';

@ChopperApi(baseUrl: '/$apiBase/client/$apiVersion')
abstract class ClientService extends ChopperService {
  static ClientService create([ChopperClient client]) =>
      _$ClientService(client);

  @Post(path: 'login')
  Future<Response> login(@Body() String body);

  @Post(path: 'pushers/set')
  Future<Response> setPusher(@Body() String body);

  @Get(path: 'register/available')
  Future<Response> registerAvailable(@Query('username') String username);

  @Get(path: 'sync')
  Future<Response> sync({
    @Query('since') @required String since,
    @Query('full_state') bool fullState = false,
    @Query('filter') String filter,
    @Query('timeout') int timeout = 0,
  });

  @Get(path: 'rooms/{roomId}/messages')
  Future<Response> roomMessages({
    @Path('roomId') @required String roomId,
    @Query('from') @required String from,
    @Query('limit') int limit,
    @Query('dir') String dir = 'b',
    @Query('filter') String filter,
  });

  @Put(path: 'rooms/{roomId}/send/{eventType}/{txnId}')
  Future<Response> send({
    @Path('roomId') @required String roomId,
    @Path('eventType') @required String eventType,
    @Path('txnId') @required String txnId,
    @Body() @required String content,
  });

  @Put(path: 'rooms/{roomId}/typing/{userId}')
  Future<Response> typing({
    @Path('roomId') @required String roomId,
    @Path('userId') @required String userId,
    @Body() @required String body,
  });

  @Post(path: 'createRoom')
  Future<Response> createRoom({
    @Body() @required String body,
  });

  @Post(path: 'logout')
  Future<Response> logout();
}
