// Copyright (C) 2019  Wilko Manger
// Copyright (C) 2019  Mathieu Velten
//
// This file is part of matrix-dart-sdk.
//
// matrix-dart-sdk is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// matrix-dart-sdk is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with matrix-dart-sdk.  If not, see <https://www.gnu.org/licenses/>.
import 'dart:async';
import 'package:chopper/chopper.dart';
import 'package:matrix_sdk/src/api/api.dart';

part 'media.chopper.dart';

@ChopperApi(baseUrl: '/$apiBase/media/$apiVersion')
abstract class MediaService extends ChopperService {
  static MediaService create([ChopperClient client]) => _$MediaService(client);

  @Get(path: 'download/{serverName}/{mediaId}')
  Future<Response<Stream<List<int>>>> download(
      @Path('serverName') String serverName, @Path('mediaId') String mediaId);

  @Post(path: 'upload')
  Future<Response> upload(
    @Body() Stream<List<int>> byteStream,
    @Header('Content-Length') String length,
    @Header('Content-Type') String contentType,
    @Query('filename') String fileName,
  );

  @Get(path: 'thumbnail/{serverName}/{mediaId}')
  Future<Response<Stream<List<int>>>> thumbnail(
    @Path('serverName') String serverName,
    @Path('mediaId') String mediaId,
    @Query('width') int width,
    @Query('height') int height,
    @Query('method') String method,
  );
}
