// Copyright (C) 2019  Wilko Manger
// Copyright (C) 2019  Mathieu Velten
//
// This file is part of matrix-dart-sdk.
//
// matrix-dart-sdk is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// matrix-dart-sdk is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with matrix-dart-sdk.  If not, see <https://www.gnu.org/licenses/>.

import 'package:meta/meta.dart';

import 'package:matrix_sdk/src/event/room/state/state_event.dart';
import '../../identifier.dart';
import '../event.dart';
import 'room_event.dart';

class Redaction extends EventContent {
  final String reason;

  Redaction({@required this.reason});

  @override
  Map<String, dynamic> toJson() => super.toJson()..addAll({'reason': reason});
}

class RedactionEvent extends RoomEvent {
  RedactionEvent(
    RoomEventArgs args, {
    @required this.content,
    @required this.redacts,
  }) : super(args);

  @override
  final Redaction content;

  final EventId redacts;

  factory RedactionEvent.fromJson(
    RoomEventArgs args,
    Map<String, dynamic> json,
  ) {
    final redacts = EventId(json['redacts']);

    var reason;
    if (json['content'].containsKey('reason')) {
      reason = json['content']['reason'];
    }

    return RedactionEvent(
      args,
      content: Redaction(reason: reason),
      redacts: redacts,
    );
  }

  @override
  Map<String, dynamic> toJson() => super.toJson()
    ..addAll({
      'redacts': redacts.toString(),
    });
}

class RedactedEvent extends RoomEvent {
  factory RedactedEvent.from({
    @required RedactionEvent redaction,
    @required RoomEvent original,
  }) {
    if (original is StateEvent) {
      return RedactedStateEvent(redaction, original);
    }
    return RedactedEvent(redaction, original);
  }

  @override
  final String matrixType;

  @override
  final EventContent content = null;

  final RedactionEvent redaction;

  final RoomEvent original;

  RedactedEvent(this.redaction, this.original)
      : matrixType = original.matrixType,
        super(RoomEventArgs.from(original));

  @override
  Map<String, dynamic> toJson() => super.toJson()
    ..addAll({
      'unsigned': {
        'redacted_because': redaction.toJson(),
      }
    });
}

class RedactedStateEvent extends StateEvent implements RedactedEvent {
  @override
  final StateEventContent content = null;

  @override
  final String matrixType;

  final RedactionEvent redaction;

  final StateEvent original;

  RedactedStateEvent(this.redaction, this.original)
      : matrixType = original.matrixType,
        super(RoomEventArgs.from(original), stateKey: original.stateKey);

  @override
  Map<String, dynamic> toJson() => super.toJson()
    ..addAll({
      'unsigned': {
        'redacted_because': redaction.toJson(),
      }
    });
}
