// Copyright (C) 2019  Wilko Manger
//
// This file is part of matrix-dart-sdk.
//
// matrix-dart-sdk is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// matrix-dart-sdk is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with matrix-dart-sdk.  If not, see <https://www.gnu.org/licenses/>.

import 'package:matrix_sdk/src/event/room/room_event.dart';
import 'package:matrix_sdk/src/event/room/state/state_event.dart';
import 'package:matrix_sdk/src/identifier.dart';
import 'package:meta/meta.dart';

class RoomCreationEvent extends StateEvent {
  @override
  final RoomCreation content;

  RoomCreationEvent(this.content, RoomEventArgs args)
      : super(args, stateKey: '');

  factory RoomCreationEvent.fromJson(RoomEventArgs args, dynamic json) {
    if (!hasStateKey(json)) {
      return null;
    }

    final content = json['content'];

    final creatorId = UserId(content['creator']);
    final federate = content['m.federate'] ?? true;
    final roomVersion = content['room_version'] ?? '1';

    var previousRoomId, previousRoomEventId;

    final predecessor = content['predecessor'];
    if (predecessor != null) {
      previousRoomId = RoomId(predecessor['room_id']);
      previousRoomEventId = EventId(predecessor['event_id']);
    }

    return RoomCreationEvent(
      RoomCreation(
        creatorId: creatorId,
        federate: federate,
        roomVersion: roomVersion,
        previousRoomId: previousRoomId,
        previousRoomLastEventId: previousRoomEventId,
      ),
      args,
    );
  }
}

class RoomCreation extends StateEventContent {
  /// User id of the creator of the room.
  ///
  /// Use [room.creator] to get the state of the
  /// creator at time of creation.
  final UserId creatorId;

  /// Whether the created room is federated.
  final bool federate;

  final String roomVersion;

  final RoomId previousRoomId;
  final EventId previousRoomLastEventId;

  RoomCreation({
    @required this.creatorId,
    @required this.federate,
    @required this.roomVersion,
    @required this.previousRoomId,
    @required this.previousRoomLastEventId,
  });

  @override
  Map<String, dynamic> toJson() => super.toJson()
    ..addAll({
      'creator': creatorId,
    });
}
