// Copyright (C) 2019  Wilko Manger
//
// This file is part of matrix-dart-sdk.
//
// matrix-dart-sdk is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// matrix-dart-sdk is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with matrix-dart-sdk.  If not, see <https://www.gnu.org/licenses/>.

import 'package:matrix_sdk/src/event/room/room_event.dart';
import 'package:matrix_sdk/src/event/room/state/state_event.dart';
import 'package:matrix_sdk/src/identifier.dart';
import 'package:meta/meta.dart';

class RoomUpgradeEvent extends StateEvent {
  @override
  final RoomUpgrade content;

  RoomUpgradeEvent(this.content, RoomEventArgs args)
      : super(args, stateKey: '');

  factory RoomUpgradeEvent.fromJson(RoomEventArgs args, dynamic json) {
    if (!hasStateKey(json)) {
      return null;
    }

    final content = json['content'];

    final body = content['body'];
    var replacementRoomId = content['replacement_room'];
    if (replacementRoomId == null) {
      return null;
    }

    replacementRoomId = RoomId(replacementRoomId);

    return RoomUpgradeEvent(
      RoomUpgrade(
        body: body,
        replacementRoomId: replacementRoomId,
      ),
      args,
    );
  }
}

class RoomUpgrade extends StateEventContent {
  final String body;

  final RoomId replacementRoomId;

  RoomUpgrade({
    @required this.body,
    @required this.replacementRoomId,
  });

  @override
  Map<String, dynamic> toJson() => super.toJson()
    ..addAll({
      'body': body,
      'replacement_room': replacementRoomId.toString(),
    });
}
